/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "holdemrender.h"

void holdemRenderPlayer(float x, float y, float z, float yaw) {
  float w, h;
  w = 0.75, h = (
    (float)HOLDEM_GAME_STATE.kagamiTileset->divY /
    (float)HOLDEM_GAME_STATE.kagamiTileset->divX
  ) * w;
  
  int i = (int32_t)(TIME_STATE.current*10)%HOLDEM_GAME_STATE.kagamiTileset->count;
  quadBuffer(HOLDEM_GAME_STATE.kagamiQuad, 0,
    -w/2, -h/2,
    HOLDEM_GAME_STATE.kagamiTileset->divisions[i].x0,
    HOLDEM_GAME_STATE.kagamiTileset->divisions[i].y1,
    w/2, h/2,
    HOLDEM_GAME_STATE.kagamiTileset->divisions[i].x1,
    HOLDEM_GAME_STATE.kagamiTileset->divisions[i].y0,
    0, 0
  );

  shaderUseTexture(GAME_STATE.shaderWorld, HOLDEM_GAME_STATE.kagamiTexture);
  shaderUsePosition(GAME_STATE.shaderWorld, x, y, z, 0, mathDeg2Rad(yaw), 0);
  primitiveDraw(HOLDEM_GAME_STATE.kagamiQuad, 0, -1);
}

void holdemRenderCard(card_t card,
  float x, float y, float z, float pitch, float yaw, float roll
) {
  tilesetdiv_t *cardFront = HOLDEM_GAME_STATE.cardTileset->divisions + card;
  quadBuffer(HOLDEM_GAME_STATE.cardPrimitive, HOLDEM_GAME_CARD_DEPTH,
    -HOLDEM_GAME_CARD_WIDTH, -HOLDEM_GAME_CARD_HEIGHT, 
    cardFront->x0, cardFront->y1,
    HOLDEM_GAME_CARD_WIDTH, HOLDEM_GAME_CARD_HEIGHT,
    cardFront->x1, cardFront->y0,
    0, 0
  );

  shaderUseTexture(GAME_STATE.shaderWorld, HOLDEM_GAME_STATE.cardTexture);
  shaderUsePosition(GAME_STATE.shaderWorld, x, y, z, pitch, yaw, roll);
  primitiveDraw(HOLDEM_GAME_STATE.cardPrimitive, 0, -1);
}

void holdemRenderWorld() {
  uint8_t i, j;
  float pitch;
  holdemplayer_t *player;
  char name[16];

  // Poker Table
  shaderUsePositionAndScale(GAME_STATE.shaderWorld,
    0, -0.01, 0.05,
    0, 0, 0,
    3.29, 3.29, 3.29
  );
  shaderUseTexture(GAME_STATE.shaderWorld, HOLDEM_GAME_STATE.tableTexture);
  primitiveDraw(HOLDEM_GAME_STATE.tablePrimitive, 0, -1);

  for(j = 0; j < HOLDEM_GAME_STATE.match.cardsFacing; j++) {
    pitch = mathDeg2Rad(-90);
    holdemRenderCard(HOLDEM_GAME_STATE.match.cards[j], j*0.2, 0, -0.2, pitch, 0, 0);
  }

  for(i = 0; i < HOLDEM_PLAYER_COUNT; i++) {
    player = HOLDEM_GAME_STATE.match.players + i;
    if(player->state & HOLDEM_STATE_FOLDED) continue;
    sprintf(name, "Player %i", i);

    shaderUseTexture(GAME_STATE.shaderWorld, HOLDEM_GAME_STATE.fontTexture);
    spriteBatchFlush(HOLDEM_GAME_STATE.fontBatch);
    shaderUsePosition(GAME_STATE.shaderWorld, -0.1,0,i*0.2, mathDeg2Rad(-90),0,0);
    fontSpriteBatchBuffer(
      HOLDEM_GAME_STATE.fontBatch,
      HOLDEM_GAME_STATE.fontTileset,
      name, FONT_RIGHT_X, FONT_CENTER_Y, 0, -1, 0.1
    );
    spriteBatchDraw(HOLDEM_GAME_STATE.fontBatch, 0, -1);

    for(j = 0; j < HOLDEM_PLAYER_HAND; j++) {
      // pitch = mathDeg2Rad(player->state & HOLDEM_STATE_SHOWING ? -90 : 90);
      holdemRenderCard(player->cards[j], j*0.2, 0, i*0.2, pitch, 0, 0);
    }
  }

  shaderUseTexture(GAME_STATE.shaderWorld, HOLDEM_GAME_STATE.fontTexture);

  spriteBatchFlush(HOLDEM_GAME_STATE.fontBatch);
  shaderUsePosition(GAME_STATE.shaderWorld, -0.1,0,-0.2, mathDeg2Rad(-90),0,0);
  fontSpriteBatchBuffer(
    HOLDEM_GAME_STATE.fontBatch,
    HOLDEM_GAME_STATE.fontTileset,
    "Dealer", FONT_RIGHT_X, FONT_CENTER_Y, 0, -1, 0.1
  );
  spriteBatchDraw(HOLDEM_GAME_STATE.fontBatch, 0, -1);

  // Draw the players
  float playerY = 0;
  holdemRenderPlayer(0, playerY, -1, 0);
  holdemRenderPlayer(-1, playerY, -0, 90);
  holdemRenderPlayer(-0.75, playerY, 0.75, -45);
  holdemRenderPlayer(0.75, playerY, 0.75, 45);
  holdemRenderPlayer(1, playerY, 0, -90);
}